from .params import PipelineParams, PipelineRecord
from ..session import session


def call_pipeline(func, p: PipelineParams):
    accessors_in = {}
    for k, v in p.dict().items():
        if k.endswith('accessor_id'):
            accessors_in[k.split('_id')[0]] = session.get_accessor(v, pop=True)
    if len(accessors_in) == 0:
        raise NoAccessorsFoundError('Expecting as least one valid accessor to run pipeline')

    # use first validated accessor ID in parameters to log info and name derived accessors
    input_description = [p.dict()[pk] for pk in p.dict().keys() if pk.endswith('accessor_id')][0]

    models = {}
    for k, v in p.dict().items():
        if k.endswith('model_id'):
            models[k.split('_id')[0]] = session.models[v]['object']

    steps = func(
        accessors_in,
        models,
        **p.dict(),
    )

    session.log_info(f'Completed {func.__name__} on {input_description}.')

    if p.keep_interm:
        interm_ids = []
        acc_interm = steps.accessors(skip_first=True, skip_last=True).items()
        for i, item in enumerate(acc_interm):
            stk, acc = item
            interm_ids.append(
                session.add_accessor(
                    acc,
                    accessor_id=f'{input_description}_{func.__name__}_step{(i + 1):02d}_{stk}'
                )
            )
    else:
        interm_ids = None

    result_id = session.add_accessor(
        steps.last,
        accessor_id=f'{p.accessor_id}_{func.__name__}_result'
    )

    return PipelineRecord(
        output_accessor_id=result_id,
        interm_accessor_ids=interm_ids,
        success=True,
        timer=steps.times
    )

class Error(Exception):
    pass

class NoAccessorsFoundError(Error):
    pass