from pathlib import Path
import unittest

from model_server.base.accessors import generate_file_accessor, write_accessor_data_to_file
from model_server.base.pipelines import segment

import model_server.conf.testing as conf
from tests.base.test_model import DummySemanticSegmentationModel

czifile = conf.meta['image_files']['czifile']
output_path = conf.meta['output_path']


class TestSegmentationPipeline(unittest.TestCase):
    def setUp(self) -> None:
        self.model = DummySemanticSegmentationModel()

    def test_call_pipeline_function(self):
        acc = generate_file_accessor(czifile['path'])
        trace = segment.segment_pipeline({'accessor': acc}, {'model': self.model}, channel=2, smooth=3)
        outfp = output_path / 'classify_pixels.tif'
        write_accessor_data_to_file(outfp, trace.last)

        import tifffile
        img = tifffile.imread(outfp)
        w = czifile['w']
        h = czifile['h']

        self.assertEqual(
            img.shape,
            (h, w),
            'Inferred image is not the expected shape'
        )

        self.assertEqual(
            img[int(w/2), int(h/2)],
            255,
            'Middle pixel is not white as expected'
        )

        self.assertEqual(
            img[0, 0],
            0,
            'First pixel is not black as expected'
        )

        interm_fps = trace.write_interm(
            output_path / 'pipelines' / 'segment_interm',
            prefix=czifile['name']
        )
        self.assertTrue([ofp.stem.split('_')[-1] for ofp in interm_fps] == ['mono', 'inference', 'smooth'])